"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.AudioTransferImplementation = void 0;
var log4js = require("log4js");
var fs = require("fs");
var child_process = require("child_process");
var mssql_service_1 = require("../db/mssql.service");
var config_1 = require("../config/config");
var file_manager_1 = require("../shared/file-manager");
var redis_service_1 = require("../db/redis.service");
var promises_shared_1 = require("../shared/promises.shared");
var exec = child_process.exec;
var billing_implementation_1 = require("../implementation/billing.implementation");
var audio_service_1 = require("../services/audio.service");
var process_implementation_1 = require("./process.implementation");
var interaction_service_1 = require("../services/interaction.service");
var request_shared_1 = require("../shared/request.shared");
var moment = require('moment');
var logger = log4js.getLogger('Task');
var CURRENT_SYSTEM_SERVICE = 'TRANSCRIPTION';
var CURRENT_IMPLEMENTATION = 'AudioTransferImplementation';
var withSentimentAnalysis = true;
var AudioTransferImplementation = (function () {
    function AudioTransferImplementation() {
    }
    AudioTransferImplementation.prototype.Execute = function (executionCode, rateRecursive) {
        var _this = this;
        logger.info("[AudioTransferImplementation::Execute] executionCode: ".concat(executionCode, " executing wave #").concat(rateRecursive));
        var executeIndex = 1;
        var ratio = config_1.RATIO_AUDIO_TRANSFER;
        var executeResult = true;
        var currentBlockPromise;
        return (0, promises_shared_1.SleepPromise)(config_1.SLEEP_AUDIO_TRANSFER)
            .then(function (_) {
            currentBlockPromise = 'BLOCK_01_GET_FROM_REDIS';
            return redis_service_1.Redis.LRange('Audios', 0, ratio - 1);
        }).then(function (result) {
            currentBlockPromise = 'BLOCK_02_LAUNCH_AUDIO_TRANSFER_IMPL';
            var promiseArray = [];
            ratio = result && result.length <= ratio ? result.length : 0;
            while (ratio >= executeIndex) {
                promiseArray.push(_this.AudioTransferExecute(executionCode));
                executeIndex++;
            }
            if (!promiseArray.length)
                promiseArray.push(Promise.resolve(true));
            return Promise.all(promiseArray);
        }).then(function (results) {
            currentBlockPromise = 'BLOCK_03_GETTING_KEYS_FOR_RECURSIVITY';
            if (results.filter(function (item) { return item === false; }).length > 0) {
                executeResult = false;
            }
            return redis_service_1.Redis.LRange('Audios', 0, ratio - 1);
        }).then(function (result) {
            currentBlockPromise = 'BLOCK_04_RELAUNCH_AUDIO_TRANSFER_IMPL';
            if (result && result.length && rateRecursive >= 1)
                return _this.Execute(executionCode, rateRecursive - 1);
            return Promise.resolve(executeResult);
        }).then(function (result) {
            currentBlockPromise = 'BLOCK_05_RESOLVING_IMPL';
            if (rateRecursive === config_1.RATE_RECURSIVE_AUDIO_TRANSFER)
                logger.info("[AudioTransferImplementation::Execute] executionCode: ".concat(executionCode, " Finishing execution wave #").concat(rateRecursive));
            var currentResult = executeResult && result ? true : false;
            return Promise.resolve(currentResult);
        }).catch(function (error) {
            var errorMessage = typeof error === 'string' ? error : (error.message || error.description);
            logger.error('[AudioTransferImplementation::Execute] executionCode:', executionCode, ', Block:', currentBlockPromise, ', getting error:', errorMessage);
            return Promise.resolve(false);
        });
    };
    AudioTransferImplementation.prototype.AudioTransferExecute = function (executionCode) {
        var _this = this;
        var storage = '';
        var currentQueueId = '';
        var processInteraction = null;
        var vcc = '';
        var processId = '';
        var interactionType = '';
        var interactionId = '';
        var interactionSection = 0;
        var interactionPart = 0;
        var currentBlockPromise = null;
        return redis_service_1.Redis.RPopLPush('Audios', 'AudiosExecute')
            .then(function (queueId) {
            currentBlockPromise = 'BLOCK_01_GETTING_DATA_FROM_PROCESS_REDIS_LIST';
            if (queueId === null || queueId === void 0 ? void 0 : queueId.length) {
                currentQueueId = queueId;
                logger.info('[AudioTransferImplementation::AudioTransferExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'START');
                var item = queueId.split(':');
                if (item === null || item === void 0 ? void 0 : item.length) {
                    vcc = item[0];
                    processId = item[1];
                    interactionType = item[2];
                    interactionId = item[3];
                    interactionSection = Number(item[4]);
                    interactionPart = Number(item[5]);
                    storage = vcc + ':' + processId;
                    logger.info('[AudioTransferImplementation::AudioTransferExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'storage :', storage);
                    return redis_service_1.Redis.GetProcessInteraction(storage, currentQueueId);
                }
                else {
                    logger.error('[AudioTransferImplementation::AudioTransferExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'QUEUE_FORMAT_ERROR');
                }
            }
            return Promise.resolve(null);
        })
            .then(function (processInteractionResult) {
            currentBlockPromise = 'BLOCK_02_SEARCHING_JSON_FILE';
            if (processInteractionResult) {
                processInteraction = processInteractionResult;
                if (processInteraction.isCancel)
                    return Promise.resolve(null);
                if (processInteraction.retranscription)
                    _this.DeleteLocalTranscriptionJSON(vcc, interactionId, interactionSection, interactionPart);
                return _this.FindLocalTranscriptionJSON(vcc, interactionId, interactionSection, interactionPart);
            }
            return Promise.resolve(false);
        })
            .then(function (foundLocalJSONTranscription) {
            currentBlockPromise = 'BLOCK_03_RESOLVING_TRANSCRIPTION_STATUS';
            if (processInteraction === null || processInteraction === void 0 ? void 0 : processInteraction.isCancel)
                return Promise.resolve(null);
            var promises = [
                Promise.resolve(foundLocalJSONTranscription)
            ];
            if (foundLocalJSONTranscription) {
                promises.push(_this.ResumeTranscription(storage, currentQueueId, processInteraction));
            }
            return Promise.all(promises);
        })
            .then(function (results) {
            currentBlockPromise = 'BLOCK_04_CREATING_TRANSCRIPTION_FOLDER';
            if (processInteraction === null || processInteraction === void 0 ? void 0 : processInteraction.isCancel)
                return Promise.resolve(null);
            var haveToResume = results[0];
            var hasResumed = results[1] ? true : false;
            if (haveToResume && hasResumed)
                throw new Error("RESUMING_TRANSCRIPTION");
            if (processInteraction) {
                logger.info('[AudioTransferImplementation::AudioTransferExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'getting GetProcessInteraction: 1');
                var baseFolder = '';
                if (processInteraction.interactionType === 'CALL') {
                    baseFolder = (processInteraction.environmentType === 'ftp' || processInteraction.environmentType === 'sftp') ? config_1.PATH_FOLDER_PROCESS_AUDIO : config_1.PATH_FOLDER_TEMP_AUDIO;
                }
                else {
                    baseFolder = config_1.TRANSCRIPTION_FOLDER;
                }
                return (0, file_manager_1.CreateFolder)(baseFolder + '/' + processInteraction.vcc);
            }
            else {
                if (currentQueueId)
                    logger.warn('[AudioTransferImplementation::AudioTransferExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'getting GetProcessInteraction: 0');
                return Promise.resolve(null);
            }
        })
            .then(function (_) {
            currentBlockPromise = 'BLOCK_05_EXECUTING_FUNCTION';
            if (processInteraction === null || processInteraction === void 0 ? void 0 : processInteraction.isCancel)
                return Promise.resolve(null);
            if (processInteraction
                &&
                    ['allegro', 'i6', 'i-ftp', 'i-sftp'].includes(processInteraction.environmentType)) {
                if (processInteraction.interactionType === 'CALL') {
                    if (['allegro', 'i6'].includes(processInteraction.environmentType)) {
                        return _this.GetIntegratedAudioFile(processInteraction, currentQueueId, executionCode);
                    }
                    else if (['i-ftp', 'i-sftp'].includes(processInteraction.environmentType)) {
                        if (processInteraction.environmentType === 'i-ftp') {
                            return _this.GetFTPAudioFile(processInteraction, currentQueueId, executionCode);
                        }
                        else {
                            return _this.GetSFTPAudioFile(processInteraction, currentQueueId, executionCode);
                        }
                    }
                }
                else {
                    return _this.GetI6TextFile(processInteraction, currentQueueId, executionCode);
                }
            }
            else if ((processInteraction === null || processInteraction === void 0 ? void 0 : processInteraction.environmentType) === 'ftp') {
                return _this.GetFTPAudioFile(processInteraction, currentQueueId, executionCode);
            }
            else if ((processInteraction === null || processInteraction === void 0 ? void 0 : processInteraction.environmentType) === 'sftp') {
                return _this.GetSFTPAudioFile(processInteraction, currentQueueId, executionCode);
            }
            return Promise.resolve(null);
        }).then(function (result) {
            currentBlockPromise = 'BLOCK_06_GETTING_AUDIO_INFO';
            if ((processInteraction === null || processInteraction === void 0 ? void 0 : processInteraction.status) === 1
                &&
                    (processInteraction === null || processInteraction === void 0 ? void 0 : processInteraction.statusDescription) === 'pendingAnalyze') {
                logger.info('[AudioTransferImplementation::AudioTransferExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'TO_MEDIA_ANALIZE');
                return _this.GetAudioInfo(processInteraction, currentQueueId, executionCode);
            }
            return Promise.resolve(null);
        })
            .then(function (_) {
            currentBlockPromise = 'BLOCK_07_SAVING_INTERACTION_ON_REDIS';
            if (processInteraction)
                return redis_service_1.Redis.SaveProcessInteraction(storage, currentQueueId, processInteraction);
            return Promise.resolve(null);
        })
            .then(function (result) {
            currentBlockPromise = 'BLOCK_08_FINISHING_QUEUE';
            var promiseArray = [];
            var billing = new billing_implementation_1.BillingImplementation();
            if (result && processInteraction && processInteraction.status === 1) {
                logger.info('[AudioTransferImplementation::AudioTransferExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'NEXT_QUEUE');
                if (processInteraction.statusDescription === 'pendingUploadRawAudio') {
                    promiseArray.push(redis_service_1.Redis.CreateQueueL('DiarizationConvert', currentQueueId));
                }
                else if (processInteraction.statusDescription === 'pendingUpload') {
                    promiseArray.push(redis_service_1.Redis.CreateQueueR('TranscriptionsUpload', currentQueueId));
                }
                promiseArray.push(redis_service_1.Redis.DeleteQueue('AudiosExecute', currentQueueId));
            }
            else if (processInteraction && processInteraction.errorMessage === 'AUDIO_NEED_SENTIMENT_ANALYSIS_BUT_PHRASES_WERE_NOT_FOUND') {
                logger.error('[AudioTransferImplementation::AudioTransferExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'AUDIO_NEED_SENTIMENT_ANALYSIS_BUT_PHRASES_WERE_NOT_FOUND');
                billing.AddCloseTranscriptionDetailToMainFile(processInteraction, processInteraction.batchTypeName, processInteraction.batchTypeId, 0, 'AUDIO_NEED_SENTIMENT_ANALYSIS_BUT_PHRASES_WERE_NOT_FOUND');
                promiseArray.push(redis_service_1.Redis.DeleteQueue('AudiosExecute', currentQueueId));
                processInteraction.retry = config_1.RETRY_AUDIO_TRANSFER;
            }
            else if (processInteraction && processInteraction.errorMessage === 'AUDIO_NEED_DIARIZATION_BUT_PHRASES_WERE_NOT_FOUND') {
                logger.error('[AudioTransferImplementation::AudioTransferExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'AUDIO_NEED_DIARIZATION_BUT_PHRASES_WERE_NOT_FOUND');
                billing.AddCloseTranscriptionDetailToMainFile(processInteraction, processInteraction.batchTypeName, processInteraction.batchTypeId, 0, 'AUDIO_NEED_DIARIZATION_BUT_PHRASES_WERE_NOT_FOUND');
                promiseArray.push(redis_service_1.Redis.DeleteQueue('AudiosExecute', currentQueueId));
                processInteraction.retry = config_1.RETRY_AUDIO_TRANSFER;
            }
            else if (processInteraction && processInteraction.retry >= config_1.RETRY_AUDIO_TRANSFER) {
                logger.error('[AudioTransferImplementation::AudioTransferExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'REMOVED_BY_RETRIES');
                billing.AddCloseTranscriptionDetailToMainFile(processInteraction, processInteraction.batchTypeName, processInteraction.batchTypeId, 0, 'REMOVED_BY_RETRIES');
                promiseArray.push(redis_service_1.Redis.DeleteQueue('AudiosExecute', currentQueueId));
            }
            else if (currentQueueId && !processInteraction) {
                logger.error('[AudioTransferImplementation::AudioTransferExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'INTERACTION_DO_NOT_EXIST');
                var interactionData = {
                    vcc: vcc,
                    processId: processId,
                    id: interactionId,
                    section: interactionSection,
                    part: interactionPart
                };
                billing.AddCloseTranscriptionDetailToMainFile(interactionData, processInteraction.batchTypeName, processInteraction.batchTypeId, 0, 'INTERACTION_DO_NOT_EXIST');
                promiseArray.push(redis_service_1.Redis.DeleteQueue('AudiosExecute', currentQueueId));
            }
            else {
                logger.error('[AudioTransferImplementation::AudioTransferExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'QUEUE_WAS_NOT_REMOVED');
            }
            if (!promiseArray.length)
                promiseArray.push(Promise.resolve(true));
            return Promise.all(promiseArray);
        }).then(function (result) {
            currentBlockPromise = 'BLOCK_09_UPDATING_PROGRESS';
            var promiseArray = [];
            if (processInteraction && processInteraction.retry >= config_1.RETRY_AUDIO_TRANSFER) {
                processInteraction.isCancel = true;
                promiseArray.push(redis_service_1.Redis.UpdateProgress(storage));
                promiseArray.push(redis_service_1.Redis.UpdateErrorProgress(storage));
                promiseArray.push(redis_service_1.Redis.DeleteProcessInteraction(storage, currentQueueId));
                promiseArray.push(redis_service_1.Redis.SaveReadyProcessInteraction(executionCode, currentQueueId, processInteraction));
                promiseArray.push(mssql_service_1.MSSql.SaveBatchError(vcc, CURRENT_SYSTEM_SERVICE, processInteraction.batchTypeId, processInteraction.batchTypeId, CURRENT_IMPLEMENTATION, processInteraction.id, null, processInteraction.errorFunction, processInteraction.errorMessage, processInteraction.errorDetail));
            }
            if (!promiseArray.length)
                promiseArray.push(Promise.resolve(true));
            return Promise.all(promiseArray);
        }).then(function (result) {
            currentBlockPromise = 'BLOCK_10_RESOLVING';
            return Promise.resolve(true);
        }).catch(function (err) {
            var errorMessage = typeof err === 'string' ? err : (err.message || err.description);
            logger.error('[AudioTransferImplementation::AudioTransferExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, ', Block:', currentBlockPromise, ', getting error:', errorMessage);
            return Promise.resolve(false);
        });
    };
    AudioTransferImplementation.prototype.FindLocalTranscriptionJSON = function (vcc, id, section, part) {
        var path = "".concat(config_1.TRANSCRIPTION_FOLDER, "/").concat(vcc, "/").concat(id, "_").concat(section, "_").concat(part, ".json");
        return (0, promises_shared_1.DummyPromise)().then(function (_) {
            logger.info('[AudioTransferImplementation::FindLocalTranscriptionJSON] file:', path);
            return (0, file_manager_1.ExistInFileSystem)(path);
        })
            .catch(function (err) {
            logger.debug('[AudioTransferImplementation::FindLocalTranscriptionJSON] Error:', err);
            return Promise.resolve(false);
        });
    };
    AudioTransferImplementation.prototype.DeleteLocalTranscriptionJSON = function (vcc, id, section, part) {
        var path = "".concat(config_1.TRANSCRIPTION_FOLDER, "/").concat(vcc, "/").concat(id, "_").concat(section, "_").concat(part, ".json");
        return (0, promises_shared_1.DummyPromise)().then(function (_) {
            logger.info('[AudioTransferImplementation::DeleteLocalTranscriptionJSON] file: ', path);
            return (0, file_manager_1.DeleteFile)(path);
        })
            .catch(function (err) {
            logger.error('[AudioTransferImplementation::DeleteLocalTranscriptionJSON] Error:', err);
            return Promise.resolve(false);
        });
    };
    AudioTransferImplementation.prototype.ResumeTranscription = function (storage, currentQueueId, pi) {
        var path = "".concat(config_1.TRANSCRIPTION_FOLDER, "/").concat(pi.vcc, "/").concat(pi.id, "_").concat(pi.section, "_").concat(pi.part, ".json");
        return (0, promises_shared_1.DummyPromise)()
            .then(function (_) {
            return (0, file_manager_1.ReadFileContent)(path, 'utf-8');
        })
            .then(function (result) {
            var _a;
            if (result) {
                try {
                    var data = JSON.parse(result);
                    var validChar_1 = 0;
                    if (data.results && data.results.length > 0) {
                        data.results.map(function (r) {
                            if (r.alternatives && r.alternatives.length > 0) {
                                validChar_1 += r.alternatives[0].content ? r.alternatives[0].content.length + 1 : 0;
                            }
                        });
                    }
                    if (!pi['fileName'])
                        pi['fileName'] = "".concat(pi.id, "_").concat(pi.section, "_").concat(pi.part, ".").concat(pi.format || 'wav');
                    pi['durationLinguisticTime'] = ((_a = data === null || data === void 0 ? void 0 : data.job) === null || _a === void 0 ? void 0 : _a.duration) ? data.job.duration : null;
                    pi['validChar'] = validChar_1;
                    pi['validWords'] = data && data.results ? data.results.length : 0;
                    pi['speechmaticsApi'] = data.speechmaticsApi;
                    pi['status'] = 3;
                    pi['statusDescription'] = pi.isDiariced === true ? 'pendingIdentifyAgent' : config_1.USAGE_SENTIMENT_ANALYSIS ? 'pendingSentimentAnalysis' : 'pendingAnalysis';
                    pi['retry'] = 0;
                    pi['uploadedDate'] = moment().format('x');
                    pi['errorFunction'] = null;
                    pi['errorMessage'] = null;
                    pi['errorDetail'] = null;
                    return Promise.resolve(true);
                }
                catch (err) {
                    logger.error('[AudioTransferImplementation::ResumeTranscription] Error: ', err);
                    throw 'CANNOT_PARSE_JSON_TRANSCRIPTION';
                }
            }
            return Promise.resolve(false);
        })
            .then(function (result) {
            if (result) {
                return redis_service_1.Redis.SaveProcessInteraction(storage, currentQueueId, pi);
            }
            return Promise.resolve(false);
        })
            .then(function (result) {
            if (result) {
                logger.debug("[AudioTransferImplementation::ResumeTranscription] Skipping ".concat(pi.id, "_").concat(pi.section, "_").concat(pi.part, " => ").concat(pi['statusDescription']));
                return redis_service_1.Redis.CreateQueueL('TranscriptionsUpload', currentQueueId);
            }
            throw 'CANNOT_SAVE_PROCESS_INTERACTION_TO_REDIS';
        })
            .then(function (result) {
            if (result) {
                var nlusToRegister = config_1.USAGE_SENTIMENT_ANALYSIS ? ["sentiment-".concat(pi.language)] : [];
                if (pi.isDiariced)
                    nlusToRegister.push("diarization-".concat(pi.language));
                var process_1 = new process_implementation_1.ProcessImplementation();
                return process_1.RunCurrentBatchesRasaModels(pi.vcc, 'TRANSCRIPTION', pi.processId, nlusToRegister, null, currentQueueId);
            }
            return Promise.resolve(result);
        })
            .then(function (saved) {
            if (saved) {
                return redis_service_1.Redis.DeleteQueue('AudiosExecute', currentQueueId);
            }
            return Promise.resolve(saved);
        })
            .catch(function (err) {
            logger.debug('[AudioTransferImplementation::ResumeTranscription] Error:', err);
            return Promise.resolve(false);
        });
    };
    AudioTransferImplementation.prototype.GetIntegratedAudioFile = function (interaction, currentQueueId, executionCode) {
        var localFilePath = '';
        var localFileName = '';
        return (0, audio_service_1.GetAudioFileFromIntegrated)(interaction.vcc, interaction.environmentType, interaction.id, interaction.section, interaction.part, null, interaction.format, 'REMOTE')
            .catch(function (err) {
            throw err && err.extraContent ? err.extraContent : 'GET_AUDIO_REQUEST_FAILED';
        })
            .then(function (result) {
            localFileName += result.audioFile.split('/').pop();
            localFilePath = config_1.PATH_FOLDER_TEMP_AUDIO + '/' + interaction.vcc + '/' + localFileName;
            return new Promise(function (resolve, reject) {
                fs.open(localFilePath, 'w', function (err, fd) {
                    if (err) {
                        logger.info('[AudioTransferImplementation::GetIntegratedAudioFile] executionCode:', executionCode, 'getting error creating local audio file:', err);
                        reject('SERVER_ERROR_CREATE_LOCAL_AUDIO_FILE_FAILED');
                    }
                    if (!Buffer.isBuffer(result.buffer)) {
                        logger.error("[AudioTransferImplementation::GetIntegratedAudioFile] executionCode: ".concat(executionCode, " getting error, the result is not a bufer object"));
                        reject('SERVER_ERROR_CANNOT_GET_AUDIO_FILE_BUFFER');
                    }
                    else {
                        fs.write(fd, result.buffer, 0, result.buffer.length, null, function (err) {
                            if (err) {
                                logger.info('[AudioTransferImplementation::GetIntegratedAudioFile] executionCode:', executionCode, 'getting error writing local audio file:', err);
                                reject('SERVER_ERROR_WRITE_LOCAL_AUDIO_FILE_FAILED');
                            }
                            fs.close(fd, function () {
                                resolve('OK');
                            });
                        });
                    }
                });
            });
        })
            .then(function (result) {
            if (result === 'OK') {
                interaction['status'] = 1;
                interaction['statusDescription'] = 'pendingUpload';
                interaction['retry'] = 0;
                interaction['filePath'] = localFilePath;
                interaction['fileName'] = localFileName;
            }
            else {
                logger.error('[AudioTransferImplementation::GetIntegratedAudioFile] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'getting errorMessage:', result);
                interaction['status'] = -1;
                interaction['retry'] = interaction['retry'] ? interaction['retry'] + 1 : 1;
                interaction['errorFunction'] = 'GetIntegratedAudioFile';
                interaction['errorMessage'] = result;
                interaction['errorDetail'] = result;
            }
            return Promise.resolve(interaction);
        })
            .catch(function (err) {
            var errorMessage = typeof err === 'string' ? err : (err.message || err.description);
            logger.error('[AudioTransferImplementation::GetIntegratedAudioFile] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'getting errorMessage:', errorMessage);
            interaction['status'] = -1;
            interaction['retry'] = interaction['retry'] ? interaction['retry'] + 1 : 1;
            interaction['errorFunction'] = 'GetAllegroAudioFile';
            interaction['errorMessage'] = errorMessage;
            interaction['errorDetail'] = JSON.stringify(err);
            return Promise.resolve(interaction);
        });
    };
    AudioTransferImplementation.prototype.GetI6TextFile = function (interaction, currentQueueId, executionCode) {
        return (0, promises_shared_1.DummyPromise)()
            .then(function (_) {
            return (0, interaction_service_1.GetJsonDataFromI6Data)(interaction);
        })
            .then(function (result) {
            if (result) {
                interaction['status'] = 1;
                interaction['statusDescription'] = 'pendingUpload';
                interaction['retry'] = 0;
                interaction['errorFunction'] = null;
                interaction['errorMessage'] = null;
                interaction['errorDetail'] = null;
            }
            else {
                interaction['status'] = -1;
                interaction['retry'] = interaction['retry'] ? interaction['retry'] + 1 : 1;
                interaction['errorFunction'] = 'GetI6TextFile';
                interaction['errorMessage'] = 'FILES_DOWNLOAD_FAILED';
                interaction['errorDetail'] = 'FILES_DOWNLOAD_FAILED';
            }
            return Promise.resolve(interaction);
        })
            .catch(function (err) {
            var errorMessage = typeof err === 'string' ? err : (err.message || err.description);
            logger.error('[GetAllegroAudioFile] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'getting errorMessage:', errorMessage);
            interaction['status'] = -1;
            interaction['retry'] = interaction['retry'] ? interaction['retry'] + 1 : 1;
            interaction['errorFunction'] = 'GetI6TextFile';
            interaction['errorMessage'] = errorMessage;
            interaction['errorDetail'] = JSON.stringify(err);
            return Promise.resolve(interaction);
        });
    };
    AudioTransferImplementation.prototype.GetFileNamePathFromInteraction = function (interaction, fileType, message) {
        var fileNamePath = '';
        var localPath = '';
        localPath = config_1.TRANSCRIPTION_FOLDER;
        localPath += '/';
        localPath += interaction.vcc;
        localPath += '/';
        fileNamePath = localPath;
        fileNamePath += this.GetFileNameFromInteraction(interaction, fileType, message);
        return fileNamePath;
    };
    AudioTransferImplementation.prototype.GetFileNameFromInteraction = function (interaction, fileType, message) {
        var interactionId = interaction.id ? interaction.id : interaction.interactionId;
        var fileName = '';
        var processInteractionId = JSON.parse(JSON.stringify(interactionId));
        var cleanedProcessInteractionId = processInteractionId.split(':').join('_');
        var localViewFileName = '';
        var localJsonFileName = '';
        localViewFileName = interaction.interactionType + '_' + interaction.campaign + '_' + interaction.folder + '_' + cleanedProcessInteractionId;
        localJsonFileName = interaction.interactionType + '_' + interaction.campaign + '_' + interaction.folder + '_' + cleanedProcessInteractionId + '.json';
        if (interaction.interactionType === 'MAIL') {
            if (message && !isNaN(message))
                localViewFileName += "_".concat(message);
            else
                localViewFileName += "_".concat(interaction.totalMessage);
            localViewFileName += '.eml';
        }
        else
            localViewFileName += '.html';
        if (fileType === 'view')
            fileName = localViewFileName;
        if (fileType === 'json')
            fileName = localJsonFileName;
        return fileName;
    };
    AudioTransferImplementation.prototype.GetI6TextFileFromUrl = function (executionCode, currentQueueId, localI6TextFileName, remoteI6TextFilePath) {
        logger.info("[AudioTransferImplementation::GetI6TextFileFromUrl] executionCode: ".concat(executionCode, " getting queueId: ").concat(currentQueueId, " url: ").concat(remoteI6TextFilePath));
        var file;
        var genericResult = {};
        return (0, promises_shared_1.DummyPromise)()
            .then(function (_) {
            var getOptions = {
                uri: remoteI6TextFilePath,
                headers: {
                    'Content-Type': 'application/x-www-form-urlencoded'
                },
                retryProtocol: false,
                timeout: 30000
            };
            return (0, request_shared_1.SimpleGetRequest)(getOptions);
        })
            .then(function (result) {
            logger.debug("[AudioTransferImplementation::GetI6TextFileFromUrl] executionCode: ".concat(executionCode, " getting queueId: ").concat(currentQueueId, " result: ").concat(result));
            if (!result)
                throw 'SERVER_ERROR_CONTENT_DOWNLOAD_FAILED';
            if (typeof result === 'object')
                return (0, file_manager_1.WriteFileContent)(localI6TextFileName, JSON.stringify(result), 'utf-8', true);
            else
                throw 'SERVER_ERROR_GET_TEXT_REQUEST_FAILED';
        })
            .then(function (result) {
            if (!result)
                throw 'SERVER_ERROR_SAVE_TEXT_CONTENT_FAILED';
            genericResult.status = true;
            genericResult.result = 'DOWNLOAD_SUCCESSFULL';
            return Promise.resolve(genericResult);
        })
            .catch(function (err) {
            var errorMessage = typeof err === 'string' ? err : err.message;
            logger.error('[AudioTransferImplementation::GetI6TextFileFromUrl] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'cannot download audio from: ', remoteI6TextFilePath, 'getting errorMessage:', errorMessage);
            genericResult.status = false;
            genericResult.error = err.message;
            return Promise.resolve(genericResult);
        });
    };
    AudioTransferImplementation.prototype.GetFTPAudioFile = function (interaction, currentQueueId, executionCode) {
        var localFilePath = '';
        var localFileName = '';
        return (0, promises_shared_1.DummyPromise)()
            .then(function (result) {
            if (interaction.id && interaction.id.length > 0) {
                localFileName = interaction.id;
                localFileName += '_';
                localFileName += interaction.section && interaction.section > 0 ? interaction.section : '0';
                localFileName += '_';
                localFileName += interaction.part && interaction.part > 0 ? interaction.part : '0';
            }
            var audioData = {
                audioId: interaction.id,
                relativePath: interaction.relativePath,
                fileName: interaction.fileName,
                relativeFileName: interaction.relativeFileName,
                section: interaction.section,
                part: interaction.part
            };
            var environment = {};
            environment.apiHost = interaction.apiHost;
            environment.apiPort = interaction.apiPort;
            environment.apiUser = interaction.apiUser;
            environment.apiPassword = interaction.apiPassword;
            environment.ftpDirectory = interaction.ftpDirectory;
            environment.type = interaction.environmentType;
            return (0, audio_service_1.GetAudioFileFromFtp)(interaction.vcc, audioData, environment)
                .catch(function (err) {
                throw err && err.extraContent ? err.extraContent : 'ERROR_AUDIO_FILE_GET_FTP';
            });
        })
            .then(function (result) {
            localFileName += '.' + result.audioFile.split('.').pop();
            if (interaction.environmentType === 'ftp' || interaction.environmentType === 'sftp')
                localFilePath = config_1.PATH_FOLDER_PROCESS_AUDIO + '/' + interaction.vcc + '/' + localFileName;
            else
                localFilePath = config_1.PATH_FOLDER_TEMP_AUDIO + '/' + interaction.vcc + '/' + localFileName;
            return new Promise(function (resolve, reject) {
                fs.open(localFilePath, 'w', function (err, fd) {
                    if (err) {
                        logger.info('[AudioTransferImplementation::GetFTPAudioFile] executionCode:', executionCode, 'getting error creating local audio file:', err);
                        reject('SERVER_ERROR_CREATE_LOCAL_AUDIO_FILE_FAILED');
                    }
                    fs.write(fd, result.buffer, 0, result.buffer.length, null, function (err) {
                        if (err) {
                            logger.info('[AudioTransferImplementation::GetFTPAudioFile] executionCode:', executionCode, 'getting error writing local audio file:', err);
                            reject('SERVER_ERROR_WRITE_LOCAL_AUDIO_FILE_FAILED');
                        }
                        fs.close(fd, function () {
                            resolve('OK');
                        });
                    });
                });
            });
        })
            .then(function (result) {
            if (result === 'OK') {
                interaction['status'] = 1;
                interaction['statusDescription'] = 'pendingAnalyze';
                interaction['retry'] = 0;
                interaction['fileName'] = localFileName;
                interaction['filePath'] = localFilePath;
                interaction['errorFunction'] = null;
                interaction['errorMessage'] = null;
                interaction['errorDetail'] = null;
            }
            else {
                logger.error('[AudioTransferImplementation::GetFTPAudioFile] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'getting errorMessage:', result);
                interaction['status'] = -1;
                interaction['retry'] = interaction['retry'] ? interaction['retry'] + 1 : 1;
                interaction['errorFunction'] = 'GetFTPAudioFile';
                interaction['errorMessage'] = result;
                interaction['errorDetail'] = result;
            }
            return Promise.resolve(interaction);
        })
            .catch(function (err) {
            var errorMessage = typeof err === 'string' ? err : (err.message || err.description);
            logger.error('[AudioTransferImplementation::GetFTPAudioFile] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'getting errorMessage:', errorMessage);
            interaction['status'] = -1;
            interaction['retry'] = interaction['retry'] ? interaction['retry'] + 1 : 1;
            interaction['errorFunction'] = 'GetFTPAudioFile';
            interaction['errorMessage'] = errorMessage;
            interaction['errorDetail'] = JSON.stringify(err);
            return Promise.resolve(interaction);
        });
    };
    AudioTransferImplementation.prototype.GetSFTPAudioFile = function (interaction, currentQueueId, executionCode) {
        var localFilePath = '';
        var localFileName = '';
        return (0, promises_shared_1.DummyPromise)()
            .then(function (result) {
            if (interaction.id && interaction.id.length > 0) {
                localFileName = interaction.id;
                localFileName += '_';
                localFileName += interaction.section && interaction.section > 0 ? interaction.section : '0';
                localFileName += '_';
                localFileName += interaction.part && interaction.part > 0 ? interaction.part : '0';
            }
            var audioData = {
                audioId: interaction.id,
                relativePath: interaction.relativePath,
                fileName: interaction.fileName,
                relativeFileName: interaction.relativeFileName,
                section: interaction.section,
                part: interaction.part
            };
            var environment = {};
            environment.apiHost = interaction.apiHost;
            environment.apiPort = interaction.apiPort;
            environment.apiUser = interaction.apiUser;
            environment.apiPassword = interaction.apiPassword;
            environment.ftpDirectory = interaction.ftpDirectory;
            environment.type = interaction.environmentType;
            return (0, audio_service_1.GetAudioFileFromSftp)(interaction.vcc, audioData, environment)
                .catch(function (err) {
                throw err && err.extraContent ? err.extraContent : 'ERROR_AUDIO_FILE_GET_SFTP';
            });
        })
            .then(function (result) {
            localFileName += '.' + result.audioFile.split('.').pop();
            if (interaction.environmentType === 'ftp' || interaction.environmentType === 'sftp')
                localFilePath = config_1.PATH_FOLDER_PROCESS_AUDIO + '/' + interaction.vcc + '/' + localFileName;
            else
                localFilePath = config_1.PATH_FOLDER_TEMP_AUDIO + '/' + interaction.vcc + '/' + localFileName;
            return new Promise(function (resolve, reject) {
                fs.open(localFilePath, 'w', function (err, fd) {
                    if (err) {
                        logger.info('[AudioTransferImplementation::GetSFTPAudioFile] executionCode:', executionCode, 'getting error creating local audio file:', err);
                        reject('SERVER_ERROR_CREATE_LOCAL_AUDIO_FILE_FAILED');
                    }
                    fs.write(fd, result.buffer, 0, result.buffer.length, null, function (err) {
                        if (err) {
                            logger.info('[AudioTransferImplementation::GetSFTPAudioFile] executionCode:', executionCode, 'getting error writing local audio file:', err);
                            reject('SERVER_ERROR_WRITE_LOCAL_AUDIO_FILE_FAILED');
                        }
                        fs.close(fd, function () {
                            resolve('OK');
                        });
                    });
                });
            });
        })
            .then(function (result) {
            if (result === 'OK') {
                interaction['status'] = 1;
                interaction['statusDescription'] = 'pendingAnalyze';
                interaction['retry'] = 0;
                interaction['fileName'] = localFileName;
                interaction['filePath'] = localFilePath;
                interaction['errorFunction'] = null;
                interaction['errorMessage'] = null;
                interaction['errorDetail'] = null;
            }
            else {
                logger.error('[AudioTransferImplementation::GetSFTPAudioFile] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'getting errorMessage:', result);
                interaction['status'] = -1;
                interaction['retry'] = interaction['retry'] ? interaction['retry'] + 1 : 1;
                interaction['errorFunction'] = 'GetSFTPAudioFile';
                interaction['errorMessage'] = result;
                interaction['errorDetail'] = result;
            }
            return Promise.resolve(interaction);
        })
            .catch(function (err) {
            var errorMessage = typeof err === 'string' ? err : (err.message || err.description);
            logger.error('[AudioTransferImplementation::GetSFTPAudioFile] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'getting errorMessage:', errorMessage);
            interaction['status'] = -1;
            interaction['retry'] = interaction['retry'] ? interaction['retry'] + 1 : 1;
            interaction['errorFunction'] = 'GetSFTPAudioFile';
            interaction['errorMessage'] = errorMessage;
            interaction['errorDetail'] = JSON.stringify(err);
            return Promise.resolve(interaction);
        });
    };
    AudioTransferImplementation.prototype.GetAudioInfo = function (interaction, currentQueueId, executionCode) {
        var needToDiarizate = false;
        var tmpPath = '';
        return (0, promises_shared_1.DummyPromise)()
            .then(function (_) {
            var cmd = '"' + config_1.MEDIAINFO_FOLDER_PATH + 'mediainfo" --Output=JSON "' + interaction.filePath + '"';
            logger.debug('[AudioTransferImplementation::GetAudioInfo] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'getting cmd1:', cmd);
            return new Promise(function (resolve, reject) {
                try {
                    exec(cmd, function (err, stdout, stderr) {
                        logger.debug('[AudioTransferImplementation::GetAudioInfo] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'getting stderr:', stderr);
                        if (err) {
                            logger.error('[AudioTransferImplementation::GetAudioInfo] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'getting errorMessage:', err);
                            reject(err);
                        }
                        else {
                            var res = typeof stdout === 'string' ? JSON.parse(stdout) : stdout;
                            resolve(res.media);
                        }
                    });
                }
                catch (e) {
                    reject(e);
                }
            });
        }).then(function (result) {
            if (result && result.track) {
                var generalInfo = result.track.find(function (t) { return t['@type'] === 'General'; });
                var audioInfo = result.track.find(function (t) { return t['@type'] === 'Audio'; });
                var minDurationToDiarizate = 60;
                var maxDurationToDiarizate = 1800;
                if (generalInfo && audioInfo) {
                    interaction.format = generalInfo.FileExtension;
                    interaction.channels = Number(audioInfo.Channels) === 1 ? 'mono' : 'stereo';
                    interaction.mediaInfoTime = audioInfo.Duration;
                    if (generalInfo.FileExtension === 'wav'
                        && Number(audioInfo.Channels) === 1
                        && Number(audioInfo.Duration) >= minDurationToDiarizate
                        && Number(audioInfo.Duration) <= maxDurationToDiarizate
                        && interaction.diarization) {
                        needToDiarizate = true;
                        tmpPath = interaction.filePath + '-TMP';
                        return (0, file_manager_1.RenameFile)(interaction.filePath, tmpPath);
                    }
                    else {
                        return Promise.resolve(true);
                    }
                }
                else {
                    interaction['retry'] = config_1.RETRY_AUDIO_TRANSFER;
                    throw 'SERVER_ERROR_CANNOT_GET_AUDIO_MEDIAINFO_ANALYSIS';
                }
            }
            else {
                interaction['retry'] = config_1.RETRY_AUDIO_TRANSFER;
                throw 'SERVER_ERROR_CANNOT_GET_AUDIO_MEDIAINFO_RESPONSE';
            }
        }).then(function (result) {
            if (config_1.USAGE_SENTIMENT_ANALYSIS)
                return mssql_service_1.MSSql.GetNLU(interaction.vcc, 'sentiment-' + interaction.language);
            else
                return Promise.resolve(null);
        }).then(function (result) {
            if (config_1.USAGE_SENTIMENT_ANALYSIS) {
                if (!result
                    ||
                        (result && !result.lastTrainedDateTime))
                    throw 'AUDIO_NEED_SENTIMENT_ANALYSIS_BUT_PHRASES_WERE_NOT_FOUND';
                return null;
            }
            else {
                return Promise.resolve(null);
            }
        }).then(function (result) {
            if (needToDiarizate) {
                return mssql_service_1.MSSql.GetNLU(interaction.vcc, 'diarization-' + interaction.language);
            }
            else {
                return Promise.resolve(null);
            }
        })
            .then(function (result) {
            if (needToDiarizate) {
                if (!result
                    ||
                        (result && !result.lastTrainedDateTime))
                    throw 'AUDIO_NEED_DIARIZATION_BUT_PHRASES_WERE_NOT_FOUND';
                var cmd_1 = config_1.FFMPEG_FOLDER_PATH + 'ffmpeg -y -i "' + tmpPath + '" -vn -ar 8000 -ac 1 -acodec pcm_s16le -f wav -fflags +bitexact -flags:a +bitexact -map_metadata -1 "' + interaction.filePath + '"';
                logger.debug('[AudioTransferImplementation::GetAudioInfo] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'getting cmd2:', cmd_1);
                return new Promise(function (resolve, reject) {
                    try {
                        exec(cmd_1, function (err, stdout, stderr) {
                            if (err) {
                                reject(err);
                            }
                            else {
                                resolve(stdout.media);
                            }
                        });
                    }
                    catch (e) {
                        reject(e);
                    }
                });
            }
            else {
                return Promise.resolve(null);
            }
        }).then(function (result) {
            if (needToDiarizate) {
                return (0, file_manager_1.DeleteFile)(tmpPath);
            }
            else {
                return Promise.resolve(null);
            }
        }).then(function (result) {
            if (needToDiarizate) {
                interaction['statusDescription'] = 'pendingUploadRawAudio';
            }
            else {
                interaction['statusDescription'] = 'pendingUpload';
            }
            return Promise.resolve(interaction);
        }).catch(function (err) {
            var errorMessage = typeof err === 'string' ? err : (err.message || err.description);
            logger.error('[AudioTransferImplementation::GetAudioInfo] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'getting errorMessage:', errorMessage);
            interaction['status'] = -1;
            interaction['retry'] = interaction['retry'] ? interaction['retry'] + 1 : 1;
            interaction['errorFunction'] = 'GetAudioInfo';
            interaction['errorMessage'] = errorMessage;
            interaction['errorDetail'] = JSON.stringify(err);
            if (errorMessage === 'AUDIO_NEED_SENTIMENT_ANALYSIS_BUT_PHRASES_WERE_NOT_FOUND' || 'AUDIO_NEED_DIARIZATION_BUT_PHRASES_WERE_NOT_FOUND')
                (0, file_manager_1.RenameFile)(tmpPath, interaction.filePath);
            return Promise.resolve(interaction);
        });
    };
    return AudioTransferImplementation;
}());
exports.AudioTransferImplementation = AudioTransferImplementation;
